"""
Custom settings for QGIS plugin
Denis Rouzaud
denis.rouzaud@gmail.com
Feb. 2013
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *


class Bool(QObject):
    valueChanged = pyqtSignal(bool)
        
    def __init__(self, pluginName, name, scope, defaultValue, options={}):
        QObject.__init__(self)
        if scope not in ("global","project"):
            raise NameError("%s is not a valid scope. Must be project or global." % scope)
        self.pluginName = pluginName
        self.name = name
        self.scope = scope
        self.defaultValue = defaultValue
        self.widget = None
        self.options = options
        self.check(defaultValue)
        
    def setWidget(self, widget):
        self.widget = widget
        
    def check(self, value):
        if type(value)!=bool:
            raise NameError("Setting %s must be a boolean." % self.name)

    def setValue(self, value):
        self.check(value)
        if self.scope == "global":
            QSettings(self.pluginName,self.pluginName).setValue(self.name, value)
        elif self.scope == "project":
            QgsProject.instance().writeEntryBool(self.pluginName, self.name, value)
        self.valueChanged.emit(value)

    def getValue(self):
        if self.scope == "global":
            return QSettings(self.pluginName,self.pluginName).value(self.name, self.defaultValue).toBool()
        elif self.scope == "project":
            return QgsProject.instance().readBoolEntry(self.pluginName, self.name, self.defaultValue)[0]
        
    def connectGuiSignal(self):
        if self.widget is None: return
        if type(self.widget)==QCheckBox or (hasattr(self.widget,"isCheckable") and self.widget.isCheckable()) :
            signal = SIGNAL("clicked()")
        else:
            raise NameError("PluginSettings does not handle %s widgets for integers for the moment" % type(self.widget))
        QObject.connect(self.widget, signal, self.setValueFromWidget)
        
    def setWidgetFromValue(self):
        if self.widget is None: return
        value = self.getValue()
        if type(self.widget)==QCheckBox or (hasattr(self.widget,"isCheckable") and self.widget.isCheckable()) :
            self.widget.setChecked(value)
        else:
            raise NameError("PluginSettings does not handle %s widgets for booleans for the moment" % type(self.widget))

    def setValueFromWidget(self,i=0):
        if type(self.widget)==QCheckBox or (hasattr(self.widget,"isCheckable") and self.widget.isCheckable()) :
            value = self.widget.isChecked()
        else:
            raise NameError("PluginSettings does not handle %s widgets for booleans for the moment" % type(self.widget))
        self.setValue(value)
