"""
Custom settings for QGIS plugin
Denis Rouzaud
denis.rouzaud@gmail.com
Feb. 2013
"""
# options:
# dialogTitle: show in color dialog

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from qgis.gui import QgsColorButton

class Color(QObject):
    valueChanged = pyqtSignal(QColor)
        
    def __init__(self, pluginName, name, scope, defaultValue, options={}):
        QObject.__init__(self)
        if scope not in ("global", "project"):
            raise NameError("%s is not a valid scope. Must be project or global." % scope)
        self.pluginName = pluginName
        self.name = name
        self.scope = scope
        self.defaultValue = defaultValue
        self.widget = None
        self.options = options
        self.check(defaultValue)

    def setWidget(self, widget):
        txt = self.options.get("dialogTitle", "")
        self.widget = QgsColorButton(widget, txt)

    def check(self, color):
        if type(color)!=QColor:
            raise NameError("Color setting %s must be a QColor." % self.name)

    def setValue(self, value):
        self.check(value)
        if self.scope == "global":
            QSettings(self.pluginName, self.pluginName).setValue(self.name, [value.red(),value.green(),value.blue()])
        elif self.scope == "project":
            QgsProject.instance().writeEntryBool(self.pluginName, self.name, QStringList(["%u" % value.red(), "%u" % value.green(), "%u" % value.blue()]))
        self.valueChanged.emit(value)

    def getValue(self):
        if self.scope == "global":
            color = QSettings(self.pluginName,self.pluginName).value(self.name, self.defaultValue).toList()
        elif self.scope == "project":
            color = QgsProject.instance().readListEntry(self.pluginName, self.name, self.defaultValue)
        if type(color)!=list or len(color)!=3:
            return self.defaultValue
        else:
            r = color[0].toInt()[0]
            g = color[1].toInt()[0]
            b = color[2].toInt()[0]
            return QColor(r, g, b)

    def connectGuiSignal(self):
        if self.widget is None: return
        self.widget.colorChanged.connect(self.setValueFromWidget)
        
    def setWidgetFromValue(self):
        if self.widget is None: return
        self.widget.setColor(self.getValue())

    def setValueFromWidget(self):
        if self.widget is None: return
        self.setValue(self.widget.color())
